@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.22 2000/09/01 10:10:08 bazsi Exp $ */
+ * $Id: io.c,v 1.24 2001/07/13 10:06:26 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -623,10 +623,12 @@ static void do_kill_fd(struct resource *r)
 }
 
 /* Initializes a file structure, and adds it to the backend's list. */
-void init_file(struct io_backend *b, struct nonblocking_fd *f, int fd)
+void init_file(struct io_backend *b, struct nonblocking_fd *f, int fd,
+	struct ol_string *fname)
 {
 	/* assert(fd); */
 	f->fd = fd;
+	f->fname = fname;
 	f->close_reason = -1; /* Invalid reason */
 	f->close_callback = NULL;
 
@@ -929,7 +931,7 @@ static int bind_unix_socket(struct address_info *c, int fd)
 		}
 		unlink(un.sun_path);
 		errno = 0;
-		if (bind(fd, (struct sockaddr *) &un, sizeof(struct sockaddr_un)) == -1) {
+		if (bind(fd, (struct sockaddr *) &un, sizeof(un) - sizeof(un.sun_path) + strlen(un.sun_path) + 1) == -1) {
 			werror("io.c: bind_unix_socket(): bind failed %z (%z)\n", 
 			       un.sun_path, strerror(errno));
 			return 0;
@@ -1167,7 +1169,7 @@ struct connect_fd *io_connect(struct io_backend *b,
 	{
 		NEW(connect_fd, fd);
 
-		init_file(b, &fd->super, s);
+		init_file(b, &fd->super, s, NULL);
 
 		fd->super.want_write = 1;
 		fd->super.write = connect_callback;
@@ -1194,7 +1196,7 @@ struct listen_fd *io_listen(struct io_backend *b,
 	{
 		NEW(listen_fd, fd);
 
-		init_file(b, &fd->super, s);
+		init_file(b, &fd->super, s, NULL);
     
 		fd->super.want_read = 1;
 		fd->super.read = listen_callback;
@@ -1205,12 +1207,13 @@ struct listen_fd *io_listen(struct io_backend *b,
 }
 
 struct io_fd *make_io_fd(struct io_backend *b,
-                         int fd)
+                         int fd,
+			 struct ol_string *fname)
 {
 	NEW(io_fd, f);
 
 	io_init_fd(fd);
-	init_file(b, &f->super, fd);
+	init_file(b, &f->super, fd, fname);
 
 	return f;
 }
@@ -1313,6 +1316,35 @@ void io_callout_flush(struct io_backend *backend)
 
 }
 
+int reopen_fd(struct nonblocking_fd *fd)
+{
+	struct stat statbuf;
+	int oldfd, newfd, oldmode;
+
+	oldfd = fd->fd;
+	if (!fd->fname || oldfd < 0)
+		return -1;
+
+	fstat(oldfd, &statbuf);
+	oldmode = fcntl(oldfd, F_GETFL);
+	if ((oldmode & O_WRONLY) || (oldmode & O_RDWR))
+		oldmode |= O_CREAT | O_APPEND;
+
+	if ((newfd = open(fd->fname->data, oldmode)) == -1)
+	{
+		werror("reopen_fd: open failedfor %S: %z\n",
+			fd->fname, strerror(errno));
+		return -1;
+	}
+
+	fchmod(newfd, statbuf.st_mode);
+	fchown(newfd, statbuf.st_uid, statbuf.st_gid);
+	dup2(newfd, oldfd);
+	close(newfd);
+
+	return 0;
+}
+
 void kill_fd(struct nonblocking_fd *fd)
 {
 	fd->super.alive = 0;
